
---------------
-- CONSTANTS --
---------------
DMI_DAMAGE = 1;
DMI_HEALING = 2;
DMI_DAMAGED = 3;
DMI_HEALED = 4;
DMI_MAX = 4;

DM_HIT = 1;
DM_CRIT = 2;
DM_DOT = 3;

DM_TABLE_A	= 1;
DM_TABLE_B	= 2;
DMT_FIGHT	= 3;
DMT_MAX		= 3

DamageMeters_Quantity_DAMAGE = 1;
DamageMeters_Quantity_HEALING = 2;
DamageMeters_Quantity_DAMAGED = 3;
DamageMeters_Quantity_HEALINGRECEIVED = 4;
DamageMeters_Quantity_DPS = 5;
DamageMeters_Quantity_HPS = 6;
DamageMeters_Quantity_DTPS = 7;
DamageMeters_Quantity_HTPS = 8;
DamageMeters_Quantity_TIME = 9;
DamageMeters_Quantity_MAX = 9;

DamageMeters_ReportQuantity_Total = -1;
DamageMeters_ReportQuantity_Leaders = -2;
DamageMeters_ReportQuantity_Events = -3;

-------------
-- GLOBALS -- 
-------------
DamageMeters_rankTable = {};

-------------------------------------------------
-- FUNCTIONS --
-------------------------------------------------

function DamageMeters_IsQuantityFight(quantity)
	return (quantity >= DamageMeters_Quantity_DPS and quantity <= DamageMeters_Quantity_HTPS);
end

function DamageMeters_IsQuantityPS(quantity)
	return DamageMeters_flags[DMFLAG_showFightAsPS] and (quantity >= DamageMeters_Quantity_DPS and quantity <= DamageMeters_Quantity_HTPS);
end

function DamageMeters_GetActiveQuant()
	if (DamageMeters_IsQuantityFight(DamageMeters_quantity)) then
		return DamageMeters_quantity - 4;
	end

	return DamageMeters_quantity;
end

function DamageMeters_GetQuantityString(quantity)
	if (DamageMeters_IsQuantityPS(quantity)) then
		return DamageMeters_PSQuantity_STRING[quantity - 4];
	end

	return DamageMeters_Quantity_STRING[quantity];
end

function DamageMeters_GetPlayerIndex(player, searchTableIndex)
	local searchTable;
	if (nil == searchTableIndex) then
		searchTable = DamageMeters_tables[DMT_ACTIVE];
	else
		searchTable = DamageMeters_tables[searchTableIndex];
	end

	local i;
	for i = 1, table.getn(searchTable) do
		if (searchTable[i].player == player) then
			return i;
		end
	end
	
	return nil;
end

DMSortTemp_sortType = 1;
DMSortTemp_quantity = 1;

function DMSortFunc_Default(a,b) 	
	local result;
	if (a[DMSortTemp_quantity] == b[DMSortTemp_quantity]) then
		result = a.player < b.player;
	else
		result = a[DMSortTemp_quantity] > b[DMSortTemp_quantity];
	end

	if (DMSortTemp_sortType == DamageMeters_Sort_INCREASING) then
		result = not result;
	end
	return result;
end

function DMSortFunc_Alphabetical(a, b)
	return a.player < b.player;
end

function DMSortFunc_Time(a, b)
	local result;
	if (a.lastTime == b.lastTime) then
		result = a.player < b.player;
	else
		result = a.lastTime < b.lastTime;
	end

	if (DMSortTemp_sortType == DamageMeters_Sort_INCREASING) then
		result = not result;
	end
	return result;
end

-- In the case of a tie, sort alphabetically.
function DamageMeters_DoSort(tableToSort, quantity, sortType)
	if (nil == sortType) then
		sortType = DamageMeters_sort;
	end
	DMSortTemp_sortType = sortType;
	DMSortTemp_quantity = quantity;

	if (DamageMeters_Sort_ALPHABETICAL == sortType) then
		table.sort(tableToSort, DMSortFunc_Alphabetical);
		return;
	end

	if (quantity == DamageMeters_Quantity_TIME) then
		table.sort(tableToSort, DMSortFunc_Time);
		return;
	elseif (DamageMeters_IsQuantityFight(quantity)) then
		DMSortTemp_quantity = quantity - 4;
		table.sort(tableToSort, DMSortFunc_Default);
		return;
	else
		table.sort(tableToSort, DMSortFunc_Default);
		return;
	end
end

function DamageMeters_DetermineRanks(tableIx)

	local count = table.getn(DamageMeters_tables[tableIx]);

	DamageMeters_rankTable = {};

	local quantity;
	for quantity = DMI_MAX, 1, -1 do
		DamageMeters_DoSort(DamageMeters_tables[tableIx], quantity, DamageMeters_Sort_DECREASING);

		local index;
		for index = 1, count do
			local playerName = DamageMeters_tables[tableIx][index].player;
			if (quantity == DMI_MAX) then
				--DMPrint("clearing for "..DamageMeters_tables[tableIx][index].player);
				DamageMeters_rankTable[playerName] = {};
			end	
			--DMPrint("quantity = "..quantity);
			DamageMeters_rankTable[playerName][quantity] = index;
		end
	end
end

function DamageMeters_DoReport(reportQuantity, destination, invert, start, count, tellTarget)
	local msg;

	if (destination == "BUFFER") then
		DamageMeters_reportBuffer = "";
	elseif (destination == "PARTY") then
		if (GetNumPartyMembers() == 0) then
			DMPrint(DM_ERROR_NOPARTY);
			return;
		end
	elseif (destination == "RAID") then
		if (GetNumRaidMembers() == 0) then
			DMPrint(DM_ERROR_NORAID);
			return;
		end
	end

	local tableIx = DMT_ACTIVE;
	if (DamageMeters_IsQuantityFight(reportQuantity)) then
		tableIx = DMT_FIGHT;
	end

	-- Determine bounds.
	local finish = start + count - 1;
	if (finish > table.getn(DamageMeters_tables[tableIx])) then
		finish = table.getn(DamageMeters_tables[tableIx]);
	end

	local step = 1;
	if (invert) then
		start = finish;
		finish = 1;
		step = -1;
	end

	------------
	-- Header --
	------------
	if (reportQuantity == DamageMeters_ReportQuantity_Total) then
		msg = string.format(DM_MSG_FULLREPORTHEADER, count, table.getn(DamageMeters_tables[tableIx]));
	elseif (reportQuantity == DamageMeters_ReportQuantity_Leaders) then
		local header = string.format(DM_MSG_LEADERREPORTHEADER, count, table.getn(DamageMeters_tables[tableIx]));
		local q;
		for q = 1, DMI_MAX do
			header = string.format("%s| %-21s", header, DamageMeters_Quantity_STRING[q]);
		end
		msg = header.."\n-------------------------------------------------------------------------------------------";
	elseif (reportQuantity == DamageMeters_ReportQuantity_Events) then
		msg = string.format(DM_MSG_EVENTREPORTHEADER, count, table.getn(DamageMeters_tables[tableIx]));
	else
		msg = string.format(DM_MSG_REPORTHEADER, DamageMeters_GetQuantityString(reportQuantity), count, table.getn(DamageMeters_tables[tableIx]));
	end
	DamageMeters_SendReportMsg(msg, destination, tellTarget);

	if (reportQuantity > 0) then
		DamageMeters_DoSort(DamageMeters_tables[tableIx], reportQuantity);
	end

	-- Calculate totals.
	local totalValue = 0;
	local index;
	local info;
	local totals = {0, 0, 0, 0, 0, 0};
	if (reportQuantity > 0) then
		if (reportQuantity <= DMI_MAX) then
			for index,info in DamageMeters_tables[tableIx] do 
				totalValue = totalValue + info[reportQuantity];
			end
		elseif(DamageMeters_IsQuantityPS(reportQuantity)) then
			for index,info in DamageMeters_tables[tableIx] do 
				totalValue = totalValue + DamageMeters_GetCombatValuePS(info[reportQuantity - 4]);
			end
		elseif (DamageMeters_IsQuantityFight(reportQuantity)) then
			for index,info in DamageMeters_tables[tableIx] do 
				totalValue = totalValue + info[reportQuantity - 4];
			end
		end
	elseif (reportQuantity == DamageMeters_ReportQuantity_Total or
			reportQuantity == DamageMeters_ReportQuantity_Leaders) then
		for index,info in DamageMeters_tables[tableIx] do 
			totals[1] = totals[1] + info[1];
			totals[2] = totals[2] + info[2];
			totals[3] = totals[3] + info[3];
			totals[4] = totals[4] + info[4];
			totals[5] = totals[5] + info.hitCount[DMI_DAMAGE];
			totals[6] = totals[6] + info.critCount[DMI_DAMAGE];
		end

		DamageMeters_DetermineRanks(tableIx);
	end


	---------------
	-- Main Loop --
	---------------
	local formatStrTotalMain_A = "%-12s %7d[%2d] %7d[%2d] %7d[%2d] %7d[%2d] %7d %7d";
	local formatStrTotalTotals = "%-12s %11d %11d %11d %11d %7d %7d";
	-- Careful here--if there isn't a space after each | it can lock up.
	local formatStrTotalMain_B = "%2d| %8d %-12s| %8d %-12s| %8d %-12s| %8d %-12s";
	local formatStrLeaderTotals= " =| %-20d | %-20d | %-20d | %-20d";
	local currentTime = GetTime();
	local i;
	local showThis;
	local visibleTotal = 0;
	--DMPrintD("start = "..start..", finish = "..finish..", quantity = "..reportQuantity);
	for i = start, finish, step do
		local value;
		msg = "";

		if (reportQuantity == DamageMeters_ReportQuantity_Total) then
			msg = string.format(formatStrTotalMain_A, DamageMeters_tables[tableIx][i].player, 
					DamageMeters_tables[tableIx][i][1], DamageMeters_rankTable[DamageMeters_tables[tableIx][i].player][1], 
					DamageMeters_tables[tableIx][i][2], DamageMeters_rankTable[DamageMeters_tables[tableIx][i].player][2], 
					DamageMeters_tables[tableIx][i][3], DamageMeters_rankTable[DamageMeters_tables[tableIx][i].player][3], 
					DamageMeters_tables[tableIx][i][4], DamageMeters_rankTable[DamageMeters_tables[tableIx][i].player][4], 
					DamageMeters_tables[tableIx][i].hitCount[DMI_DAMAGE], DamageMeters_tables[tableIx][i].critCount[DMI_DAMAGE]);
		elseif(reportQuantity == DamageMeters_ReportQuantity_Leaders) then
			local leaders = {};
			local leaderIndexes = {};
			local qIx;
			for qIx = 2, DMI_MAX do
				local leaderName, rank;
				for leaderName, rank in DamageMeters_rankTable do
					--DMPrint(i.." "..leaderName.." "..rank[qIx]);
					if (rank[qIx] == i) then
						leaders[qIx] = leaderName;
						leaderIndexes[qIx] = DamageMeters_GetPlayerIndex(leaderName);
						break;
					end
				end
			end

			msg = string.format(formatStrTotalMain_B, i, DamageMeters_tables[tableIx][i][1], DamageMeters_tables[tableIx][i].player,	DamageMeters_tables[tableIx][leaderIndexes[2]][2], leaders[2], DamageMeters_tables[tableIx][leaderIndexes[3]][3], leaders[3], DamageMeters_tables[tableIx][leaderIndexes[4]][4], leaders[4]); 
		elseif (reportQuantity == DamageMeters_ReportQuantity_Events) then
			DamageMeters_DumpPlayerEvents(DamageMeters_tables[tableIx][i].player, destination, true, i, totals);
		else
			if (DamageMeters_Quantity_TIME == reportQuantity) then
				local idleTime = currentTime - DamageMeters_tables[tableIx][i].lastTime;
				msg = string.format("#%.2d: %-16s  %d:%.02d", i, DamageMeters_tables[tableIx][i].player, idleTime / 60, math.mod(idleTime, 60));
			elseif (DamageMeters_IsQuantityPS(reportQuantity)) then
				local dps = DamageMeters_GetCombatValuePS(DamageMeters_tables[tableIx][i][reportQuantity - 4]);
				visibleTotal = visibleTotal + dps;
				if (dps > 0) then
					msg = string.format("#%.2d: %-16s  %.1f", i, DamageMeters_tables[tableIx][i].player, dps);
				end
			elseif (DamageMeters_IsQuantityFight(reportQuantity)) then
				value = DamageMeters_tables[tableIx][i][reportQuantity - 4];
				visibleTotal = visibleTotal + value;
				if (value > 0) then
					local percentage = (totalValue > 0) and (100 * value / totalValue) or 0;
					msg = string.format("#%.2d:  %.2f%%  %-16s  %d", i, percentage, DamageMeters_tables[tableIx][i].player, value);
				end
			else
				value = DamageMeters_tables[tableIx][i][reportQuantity];
				visibleTotal = visibleTotal + value;
				if (value > 0) then
					local percentage = (totalValue > 0) and (100 * value / totalValue) or 0;
					msg = string.format("#%.2d:  %.2f%%  %-16s  %d", i, percentage, DamageMeters_tables[tableIx][i].player, value);
				end
			end
		end
		
		if (msg ~= "") then
			DamageMeters_SendReportMsg(msg, destination, tellTarget);
		end
	end

	------------
	-- Totals --
	------------
	if (reportQuantity == DamageMeters_ReportQuantity_Total or
		reportQuantity == DamageMeters_ReportQuantity_Leaders) then

		if (reportQuantity == DamageMeters_ReportQuantity_Total) then
			msg = string.format(formatStrTotalTotals, DM_MSG_TOTAL, totals[1], totals[2], totals[3], totals[4], totals[5], totals[6]);
		else
			msg = string.format(formatStrLeaderTotals, totals[1], totals[2], totals[3], totals[4], totals[5], totals[6]);
		end
		DamageMeters_SendReportMsg(msg, destination, tellTarget);

		-- Print a list of contributors.
		if (DamageMeters_flags[DMFLAG_haveContributors]) then
			msg = string.format(DM_MSG_COLLECTORS, UnitName("Player"));
			for contrib, unused in DamageMeters_contributorList do
				msg = msg..", "..contrib;
			end
			DamageMeters_SendReportMsg(msg, destination, tellTarget);
		end
	elseif (DamageMeters_Quantity_DPS == reportQuantity) then
		msg = string.format(DM_MSG_COMBATDURATION, DamageMeters_combatEndTime - DamageMeters_combatStartTime);
		DamageMeters_SendReportMsg(msg, destination, tellTarget);
	end

	if (reportQuantity > 0) then
		if (DamageMeters_Quantity_TIME == reportQuantity) then
			-- No total.
		elseif (DamageMeters_Quantity_DPS == reportQuantity) then
			msg = string.format(DM_MSG_REPORTTOTALDPS, totalValue, visibleTotal);
			DamageMeters_SendReportMsg(msg, destination, tellTarget);
		else
			msg = string.format(DM_MSG_REPORTTOTAL, totalValue, visibleTotal);
			DamageMeters_SendReportMsg(msg, destination, tellTarget);
		end
	end
end

function DamageMeters_DumpPlayerEvents(player, destination, bIncludeName, index, totals, singleQuantity)
	local tellTarget = nil;

	local playerIndex = DamageMeters_GetPlayerIndex(player, DMT_VISIBLE);
	local playerStruct = DamageMeters_tables[DMT_VISIBLE][playerIndex].events;
	if (nil == playerStruct) then
		-- Makes spam.
		--DMPrint("Player "..player.." not found.");
		return;
	end

	local prefix = "";
	if (bIncludeName) then
		if (index) then
			str = index..": "..player..":";
		else
			str = player..":";
		end
		prefix = "  ";
		DamageMeters_SendReportMsg(str, destination, tellTarget);
	end

	if (totals == nil) then
		totals = {};
		local ii;
		for ii = 1, table.getn(DamageMeters_tables[DMT_VISIBLE]) do
			local q;
			for q = 1, DMI_MAX do
				if (totals[q] == nil) then
					totals[q] = 0;
				end
				totals[q] = totals[q] + DamageMeters_tables[DMT_VISIBLE][ii][q];
			end
		end
	end

	if (DamageMeters_debugEnabled) then
		if (playerIndex and DamageMeters_tables[DMT_VISIBLE][playerIndex].firstMsg and DamageMeters_tables[DMT_VISIBLE][playerIndex].firstMsg["event"]) then
			local msg = "Last message: "..DamageMeters_tables[DMT_VISIBLE][playerIndex].firstMsg.event.." ("..DamageMeters_tables[DMT_VISIBLE][playerIndex].firstMsg.desc..")";
			DamageMeters_SendReportMsg(msg, destination, tellTarget);
			msg = "  "..DamageMeters_tables[DMT_VISIBLE][playerIndex].firstMsg.fullMsg.."\n";
			DamageMeters_SendReportMsg(msg, destination, tellTarget);
		end
	end

	local headerDest = destination;
	if (headerDest == "TOOLTIP") then
		headerDest = "TOOLTIP_TITLE";
	end

	local quantity, quantityStruct;
	if (singleQuantity) then
		quantity = singleQuantity;
		quantityStruct = playerStruct[singleQuantity];
		if (quantityStruct) then
			DamageMeters_DumpQuantityEvents(quantity, quantityStruct, totals, headerDest, destination, tellTarget, prefix);
		end
	else
		for quantity, quantityStruct in playerStruct do
			DamageMeters_DumpQuantityEvents(quantity, quantityStruct, totals, headerDest, destination, tellTarget, prefix);
		end
	end

	DamageMeters_SendReportMsg("", destination, tellTarget);
end

function DamageMeters_DumpQuantityEvents(quantity, quantityStruct, totals, headerDest, destination, tellTarget, prefix)
	-- Calculate total.
	local total = 0;
	for spell, spellStruct in quantityStruct.spellTable do
		-- Spells with "*" at the end are duplicates, and are not to be counted towards the total.
		if (string.sub(spell, -1) ~= "*") then
			total = total + spellStruct.value;
		end
	end
	local percentageOfTotal = (totals[quantity] > 0) and (100 * total / totals[quantity]) or (0);
	str = string.format("%s = %s (%.1f%%)", prefix..DamageMeters_Quantity_STRING[quantity], total, percentageOfTotal);
	DamageMeters_SendReportMsg(str, headerDest, tellTarget);

	-- Cannot sort spells, as that table is indexed by string.
	if (quantityStruct.dirty) then
		DamageMeters_BuildSpellHash(quantityStruct);
	end

	-- Print
	for hashIndex = 1, table.getn(quantityStruct.hash) do
		local spell = quantityStruct.hash[hashIndex].spell;
		spellStruct = quantityStruct.spellTable[spell];
		--DMPrint("Hash "..hashIndex.." = "..spell);

		local percentageOfTotal = (total > 0) and (100 * spellStruct.value / total) or (0);
		local critPercentage = (spellStruct.counts[DM_HIT] > 0) and (100 * spellStruct.counts[DM_CRIT] / spellStruct.counts[DM_HIT]) or 0.0;
		local average = string.format("%.1f Avg", (spellStruct.counts[DM_HIT] and spellStruct.value / spellStruct.counts[DM_HIT] or 0));

		str = string.format(prefix.."  %s = %d (%.1f%%)  %d/%d %.1f%% %s", spell, spellStruct.value, percentageOfTotal, spellStruct.counts[DM_CRIT], spellStruct.counts[DM_HIT], critPercentage, average);

		-- Show resistance info.
		if (quantity == DMI_DAMAGED and 
			(spellStruct.damageType <= DM_DMGTYPE_RESISTMAX or spellStruct.resistanceSum > 0)) then
			local avgResist = spellStruct.resistanceSum / spellStruct.resistanceCount;
			str = string.format("%s (%d %s)", str, avgResist, DM_DMGTYPENAMES[spellStruct.damageType]);
		end

		DamageMeters_SendReportMsg(str, destination, tellTarget);
	end
end

function DamageMeters_BuildSpellHash(quantityStruct)
	local hashIndex = 1;
	for spell, spellStruct in quantityStruct.spellTable do
		quantityStruct.hash[hashIndex] = {};
		quantityStruct.hash[hashIndex].spell = spell;
		quantityStruct.hash[hashIndex].value = spellStruct.value;
		hashIndex = hashIndex + 1;
	end

	table.sort(quantityStruct.hash, 
		function(a,b) 
			return a.value > b.value;
		end
		);

	quantityStruct.dirty = false;
end